@@ -1,7 +1,7 @@
 package Complete::Bash;
 
-our $DATE = '2016-01-21'; # DATE
-our $VERSION = '0.25'; # VERSION
+our $DATE = '2016-02-02'; # DATE
+our $VERSION = '0.26'; # VERSION
 
 use 5.010001;
 use strict;
@@ -12,7 +12,6 @@ our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(
                        point
                        parse_cmdline
-                       parse_options
                        format_completion
                );
 
@@ -20,10 +19,7 @@ our %SPEC;
 
 $SPEC{':package'} = {
     v => 1.1,
-    summary => 'Completion module for bash shell',
-    links => [
-        {url => 'pm:Complete'},
-    ],
+    summary => 'Completion routines for bash shell',
 };
 
 sub _expand_tilde {
@@ -124,34 +120,81 @@ $SPEC{parse_cmdline} = {
     summary => 'Parse shell command-line for processing by completion routines',
     description => <<'_',
 
-This function basically converts COMP_LINE (str) and COMP_POINT (int) into
-something like (but not exactly the same as) COMP_WORDS (array) and COMP_CWORD
-(int) that bash supplies to shell functions.
+This function basically converts `COMP_LINE` (str) and `COMP_POINT` (int) into
+something like (but not exactly the same as) `COMP_WORDS` (array) and
+`COMP_CWORD` (int) that bash supplies to shell functions.
 
 The differences with bash are (these differences are mostly for parsing
-convenience for programs that use this routine):
+convenience for programs that use this routine; this comparison is made against
+bash versions 4.2-4.3):
 
-1) quotes and backslashes are stripped (bash's COMP_WORDS contains all the
-quotes and backslashes);
+1) quotes and backslashes are stripped (bash's `COMP_WORDS` contains all the
+   quotes and backslashes);
 
-2) variables are substituted with their values from environment variables except
-for the current word (COMP_WORDS[COMP_CWORD]) (bash does not perform variable
-substitution for COMP_WORDS). However, note that special shell variables that
-are not environment variables like `$0`, `$_`, `$IFS` will not be replaced
-correctly because bash does not export those variables for us.
+2) quoted phrase that contain spaces, or phrase that contains escaped spaces is
+   parsed as a single word. For example:
 
-3) tildes (~) are expanded with user's home directory except for the current
-word (bash does not perform tilde expansion for COMP_WORDS);
+    command "First argument" Second\ argument
 
-4) no word-breaking characters aside from whitespaces and `=` are currently used
-(bash uses COMP_WORDBREAKS which by default also include `:`, `;`, and so on).
-This is done for convenience of parsing of Getopt::Long-based applications. More
-word-breaking characters might be used in the future, e.g. when we want to
-handle complex bash statements like pipes, redirection, etc.
+   bash would split it as (represented as Perl):
+
+    ["command", "\"First", "argument\"", "Second\\", "argument"]
+
+   which is not very convenient. We parse it into:
+
+    ["command", "First argument", "Second argument"]
+
+3) variables are substituted with their values from environment variables except
+   for the current word (`COMP_WORDS[COMP_CWORD]`) (bash does not perform
+   variable substitution for `COMP_WORDS`). However, note that special shell
+   variables that are not environment variables like `$0`, `$_`, `$IFS` will not
+   be replaced correctly because bash does not export those variables for us.
+
+4) tildes (`~`) are expanded with user's home directory except for the current
+   word (bash does not perform tilde expansion for `COMP_WORDS`);
 
 Caveats:
 
-* Due to the way bash parses the command line, the two below are equivalent:
+* Like bash, we group non-whitespace word-breaking characters into its own word.
+  By default `COMP_WORDBREAKS` is:
+
+    "'@><=;|&(:
+
+  So if raw command-line is:
+
+    command --foo=bar http://example.com:80 mail@example.org Foo::Bar
+
+  then the parse result will be:
+
+    ["command", "--foo", "=", "bar", "http", ":", "//example.com", ":", "80", "Foo", "::", "Bar"]
+
+  which is annoying sometimes. But we follow bash here so we can more easily
+  accept input from a joined `COMP_WORDS` if we write completion bash functions,
+  e.g. (in the example, `foo` is a Perl script):
+
+    _foo ()
+    {
+        local words=(${COMP_CWORDS[@]})
+        # add things to words, etc
+        local point=... # calculate the new point
+        COMPREPLY=( `COMP_LINE="foo ${words[@]}" COMP_POINT=$point foo` )
+    }
+
+  To avoid these word-breaking characters to be split/grouped, we can escape
+  them with backslash or quote them, e.g.:
+
+    command "http://example.com:80" Foo\:\:Bar
+
+  which bash will parse as:
+
+    ["command", "\"http://example.com:80\"", "Foo\\:\\:Bar"]
+
+  and we parse as:
+
+    ["command", "http://example.com:80", "Foo::Bar"]
+
+* Due to the way bash parses the command line (see above), the two below are
+  equivalent:
 
     % cmd --foo=bar
     % cmd --foo = bar
@@ -175,6 +218,7 @@ _
         },
         opts => {
             summary => 'Options',
+            schema => 'hash*',
             description => <<'_',
 
 Optional. Known options:
@@ -195,8 +239,8 @@ _
 
 Return a 2-element array: `[$words, $cword]`. `$words` is array of str,
 equivalent to `COMP_WORDS` provided by bash to shell functions. `$cword` is an
-integer, equivalent to `COMP_CWORD` provided by bash to shell functions. The
-word to be completed is at `$words->[$cword]`.
+integer, roughly equivalent to `COMP_CWORD` provided by bash to shell functions.
+The word to be completed is at `$words->[$cword]`.
 
 Note that COMP_LINE includes the command name. If you want the command-line
 arguments only (like in `@ARGV`), you need to strip the first element from
@@ -229,15 +273,15 @@ sub parse_cmdline {
     my $chunk;
     my $add_blank;
     my $is_cur_word;
-    $line =~ s!(                                                 # 1) everything
-                  (")((?: \\\\|\\"|[^"])*)(?:"|\z)(\s*)       |  # 2) open "  3) content  4) space after
-                  (')((?: \\\\|\\'|[^'])*)(?:'|\z)(\s*)       |  # 5) open '  6) content  7) space after
-                  ((?: \\\\|\\"|\\'|\\=|\\\s|[^"'=\s])+)(\s*) |  # 8) unquoted word  9) space after
-                  = |
+    $line =~ s!(                                                         # 1) everything
+                  (")((?: \\\\|\\"|[^"])*)(?:"|\z)(\s*)               |  #  2) open "  3) content  4) space after
+                  (')((?: \\\\|\\'|[^'])*)(?:'|\z)(\s*)               |  #  5) open '  6) content  7) space after
+                  ((?: \\\\|\\"|\\'|\\=|\\\s|[^"'@><=|&\(:\s])+)(\s*) |  #  8) unquoted word  9) space after
+                  ([\@><=|&\(:]+) |                                      #  10) non-whitespace word-breaking characters
                   \s+
               )!
                   $pos += length($1);
-                  #say "D: \$1=<$1> \$2=<$3> \$3=<$3> \$4=<$4> \$5=<$5> \$6=<$6> \$7=<$7> \$8=<$8> \$9=<$9>";
+                  #say "D: \$1=<$1> \$2=<$3> \$3=<$3> \$4=<$4> \$5=<$5> \$6=<$6> \$7=<$7> \$8=<$8> \$9=<$9> \$10=<$10>";
                   #say "D:<$1> pos=$pos, point=$point, cword=$cword, after_ws=$after_ws";
 
                   if ($2 || $5 || defined($8)) {
@@ -281,9 +325,9 @@ sub parse_cmdline {
                       }
                       $after_ws = ($2 ? $4 : $5 ? $7 : $9) ? 1:0;
 
-                  } elsif ($1 eq '=') {
-                      # equal sign as word-breaking character
-                      push @words, '=';
+                  } elsif ($10) {
+                      # non-whitespace word-breaking characters
+                      push @words, $10;
                       $after_ws = 1;
                   } else {
                       # whitespace
@@ -297,143 +341,6 @@ sub parse_cmdline {
     [\@words, $cword];
 }
 
-$SPEC{parse_options} = {
-    v => 1.1,
-    summary => 'Parse command-line for options and arguments, '.
-        'more or less like Getopt::Long',
-    description => <<'_',
-
-Parse command-line into words using `parse_cmdline()` then separate options and
-arguments. Since this routine does not accept `Getopt::Long` (this routine is
-meant to be a generic option parsing of command-lines), it uses a few simple
-rules to server the common cases:
-
-* After `--`, the rest of the words are arguments (just like Getopt::Long).
-
-* If we get something like `-abc` (a single dash followed by several letters) it
-  is assumed to be a bundle of short options.
-
-* If we get something like `-MData::Dump` (a single dash, followed by a letter,
-  followed by some letters *and* non-letters/numbers) it is assumed to be an
-  option (`-M`) followed by a value.
-
-* If we get something like `--foo` it is a long option. If the next word is an
-  option (starts with a `-`) then it is assumed that this option does not have
-  argument. Otherwise, the next word is assumed to be this option's value.
-
-* Otherwise, it is an argument (that is, permute is assumed).
-
-_
-
-    args => {
-        cmdline => {
-            summary => 'Command-line, defaults to COMP_LINE environment',
-            schema => 'str*',
-        },
-        point => {
-            summary => 'Point/position to complete in command-line, '.
-                'defaults to COMP_POINT',
-            schema => 'int*',
-        },
-        words => {
-            summary => 'Alternative to passing `cmdline` and `point`',
-            schema => ['array*', of=>'str*'],
-            description => <<'_',
-
-If you already did a `parse_cmdline()`, you can pass the words result (the first
-element) here to avoid calling `parse_cmdline()` twice.
-
-_
-        },
-        cword => {
-            summary => 'Alternative to passing `cmdline` and `point`',
-            schema => ['array*', of=>'str*'],
-            description => <<'_',
-
-If you already did a `parse_cmdline()`, you can pass the cword result (the
-second element) here to avoid calling `parse_cmdline()` twice.
-
-_
-        },
-    },
-    result => {
-        schema => 'hash*',
-    },
-};
-sub parse_options {
-    my %args = @_;
-
-    my ($words, $cword) = @_;
-    if ($args{words}) {
-        ($words, $cword) = ($args{words}, $args{cword});
-    } else {
-        ($words, $cword) = @{parse_cmdline($args{cmdline}, $args{point})};
-    }
-
-    my @types;
-    my %opts;
-    my @argv;
-    my $type;
-    $types[0] = 'command';
-    my $i = 1;
-    while ($i < @$words) {
-        my $word = $words->[$i];
-        if ($word eq '--') {
-            if ($i == $cword) {
-                $types[$i] = 'opt_name';
-                $i++; next;
-            }
-            $types[$i] = 'separator';
-            for ($i+1 .. @$words-1) {
-                $types[$_] = 'arg,' . @argv;
-                push @argv, $words->[$_];
-            }
-            last;
-        } elsif ($word =~ /\A-(\w*)\z/) {
-            $types[$i] = 'opt_name';
-            for (split '', $1) {
-                push @{ $opts{$_} }, undef;
-            }
-            $i++; next;
-        } elsif ($word =~ /\A-([\w?])(.*)/) {
-            $types[$i] = 'opt_name';
-            # XXX currently not completing option value
-            push @{ $opts{$1} }, $2;
-            $i++; next;
-        } elsif ($word =~ /\A--(\w[\w-]*)\z/) {
-            $types[$i] = 'opt_name';
-            my $opt = $1;
-            $i++;
-            if ($i < @$words) {
-                if ($words->[$i] eq '=') {
-                    $types[$i] = 'separator';
-                    $i++;
-                }
-                if ($words->[$i] =~ /\A-/) {
-                    push @{ $opts{$opt} }, undef;
-                    next;
-                }
-                $types[$i] = 'opt_val';
-                push @{ $opts{$opt} }, $words->[$i];
-                $i++; next;
-            }
-        } else {
-            $types[$i] = 'arg,' . @argv;
-            push @argv, $word;
-            $i++; next;
-        }
-    }
-
-    return {
-        opts      => \%opts,
-        argv      => \@argv,
-        cword     => $cword,
-        words     => $words,
-        word_type => $types[$cword],
-        #_types    => \@types,
-    };
-}
-
 $SPEC{format_completion} = {
     v => 1.1,
     summary => 'Format completion for output (for shell)',
@@ -582,7 +489,7 @@ sub format_completion {
 }
 
 1;
-# ABSTRACT: Completion module for bash shell
+# ABSTRACT: Completion routines for bash shell
 
 __END__
 
@@ -592,14 +499,18 @@ __END__
 
 =head1 NAME
 
-Complete::Bash - Completion module for bash shell
+Complete::Bash - Completion routines for bash shell
 
 =head1 VERSION
 
-This document describes version 0.25 of Complete::Bash (from Perl distribution Complete-Bash), released on 2016-01-21.
+This document describes version 0.26 of Complete::Bash (from Perl distribution Complete-Bash), released on 2016-02-02.
 
 =head1 DESCRIPTION
 
+This module provides routines related to tab completion in bash shell.
+
+=head2 About programmable completion in bash
+
 Bash allows completion to come from various sources. The simplest is from a list
 of words (C<-W>):
 
@@ -624,10 +535,10 @@ possible completion:
  % foo <Tab>
  --help  --verbose  --version
 
-And yet another source is an external command (including, a Perl script). The
-command receives two environment variables: C<COMP_LINE> (string, raw
-command-line) and C<COMP_POINT> (integer, cursor location). Program must split
-C<COMP_LINE> into words, find the word to be completed, complete that, and
+And yet another source is an external command (C<-C>) including, from a Perl
+script. The command receives two environment variables: C<COMP_LINE> (string,
+raw command-line) and C<COMP_POINT> (integer, cursor location). Program must
+split C<COMP_LINE> into words, find the word to be completed, complete that, and
 return the list of words one per-line to STDOUT. An example:
 
  % cat foo-complete
@@ -642,7 +553,15 @@ return the list of words one per-line to STDOUT. An example:
  % foo --v<Tab>
  --verbose --version
 
-This module provides routines for you to be doing the above.
+=head2 About the routines in this module
+
+First of all, C<parse_cmdline()> is the function to parse raw command-line (such
+as what you get from bash in C<COMP_LINE> environment variable) into words. This
+makes it easy for the other functions to generate completion answer. See the
+documentation for that function for more details.
+
+C<format_completion> is what you use to format completion answer structure for
+bash.
 
 =head1 FUNCTIONS
 
@@ -722,36 +641,83 @@ Return value: Formatted string (or array, if `as` is set to `array`) (str|array)
 
 Parse shell command-line for processing by completion routines.
 
-This function basically converts COMP_LINE (str) and COMP_POINT (int) into
-something like (but not exactly the same as) COMP_WORDS (array) and COMP_CWORD
-(int) that bash supplies to shell functions.
+This function basically converts C<COMP_LINE> (str) and C<COMP_POINT> (int) into
+something like (but not exactly the same as) C<COMP_WORDS> (array) and
+C<COMP_CWORD> (int) that bash supplies to shell functions.
 
 The differences with bash are (these differences are mostly for parsing
-convenience for programs that use this routine):
+convenience for programs that use this routine; this comparison is made against
+bash versions 4.2-4.3):
 
-1) quotes and backslashes are stripped (bash's COMP_WORDS contains all the
-quotes and backslashes);
+1) quotes and backslashes are stripped (bash's C<COMP_WORDS> contains all the
+   quotes and backslashes);
 
-2) variables are substituted with their values from environment variables except
-for the current word (COMP_WORDS[COMP_CWORD]) (bash does not perform variable
-substitution for COMP_WORDS). However, note that special shell variables that
-are not environment variables like C<$0>, C<$_>, C<$IFS> will not be replaced
-correctly because bash does not export those variables for us.
+2) quoted phrase that contain spaces, or phrase that contains escaped spaces is
+   parsed as a single word. For example:
 
-3) tildes (~) are expanded with user's home directory except for the current
-word (bash does not perform tilde expansion for COMP_WORDS);
+ command "First argument" Second\ argument
 
-4) no word-breaking characters aside from whitespaces and C<=> are currently used
-(bash uses COMP_WORDBREAKS which by default also include C<:>, C<;>, and so on).
-This is done for convenience of parsing of Getopt::Long-based applications. More
-word-breaking characters might be used in the future, e.g. when we want to
-handle complex bash statements like pipes, redirection, etc.
+   bash would split it as (represented as Perl):
+
+ ["command", "\"First", "argument\"", "Second\\", "argument"]
+
+   which is not very convenient. We parse it into:
+
+ ["command", "First argument", "Second argument"]
+
+3) variables are substituted with their values from environment variables except
+   for the current word (C<COMP_WORDS[COMP_CWORD]>) (bash does not perform
+   variable substitution for C<COMP_WORDS>). However, note that special shell
+   variables that are not environment variables like C<$0>, C<$_>, C<$IFS> will not
+   be replaced correctly because bash does not export those variables for us.
+
+4) tildes (C<~>) are expanded with user's home directory except for the current
+   word (bash does not perform tilde expansion for C<COMP_WORDS>);
 
 Caveats:
 
 =over
 
-=item * Due to the way bash parses the command line, the two below are equivalent:
+=item * Like bash, we group non-whitespace word-breaking characters into its own word.
+By default C<COMP_WORDBREAKS> is:
+
+"'@><=;|&(:
+
+So if raw command-line is:
+
+command --foo=bar http://example.com:80 mail@example.org Foo::Bar
+
+then the parse result will be:
+
+["command", "--foo", "=", "bar", "http", ":", "//example.com", ":", "80", "Foo", "::", "Bar"]
+
+which is annoying sometimes. But we follow bash here so we can more easily
+accept input from a joined C<COMP_WORDS> if we write completion bash functions,
+e.g. (in the example, C<foo> is a Perl script):
+
+I<foo ()
+{
+    local words=(${COMP>CWORDS[@]})
+    # add things to words, etc
+    local point=... # calculate the new point
+    COMPREPLY=( C<COMP_LINE="foo ${words[@]}" COMP_POINT=$point foo> )
+}
+
+To avoid these word-breaking characters to be split/grouped, we can escape
+them with backslash or quote them, e.g.:
+
+command "http://example.com:80" Foo\:\:Bar
+
+which bash will parse as:
+
+["command", "\"http://example.com:80\"", "Foo\:\:Bar"]
+
+and we parse as:
+
+["command", "http://example.com:80", "Foo::Bar"]
+
+=item * Due to the way bash parses the command line (see above), the two below are
+equivalent:
 
 % cmd --foo=bar
 % cmd --foo = bar
@@ -797,84 +763,14 @@ Return value:  (array)
 
 Return a 2-element array: C<[$words, $cword]>. C<$words> is array of str,
 equivalent to C<COMP_WORDS> provided by bash to shell functions. C<$cword> is an
-integer, equivalent to C<COMP_CWORD> provided by bash to shell functions. The
-word to be completed is at C<< $words-E<gt>[$cword] >>.
+integer, roughly equivalent to C<COMP_CWORD> provided by bash to shell functions.
+The word to be completed is at C<< $words-E<gt>[$cword] >>.
 
 Note that COMP_LINE includes the command name. If you want the command-line
 arguments only (like in C<@ARGV>), you need to strip the first element from
 C<$words> and reduce C<$cword> by 1.
 
 
-=head2 parse_options(%args) -> [status, msg, result, meta]
-
-Parse command-line for options and arguments, more or less like Getopt::Long.
-
-Parse command-line into words using C<parse_cmdline()> then separate options and
-arguments. Since this routine does not accept C<Getopt::Long> (this routine is
-meant to be a generic option parsing of command-lines), it uses a few simple
-rules to server the common cases:
-
-=over
-
-=item * After C<-->, the rest of the words are arguments (just like Getopt::Long).
-
-=item * If we get something like C<-abc> (a single dash followed by several letters) it
-is assumed to be a bundle of short options.
-
-=item * If we get something like C<-MData::Dump> (a single dash, followed by a letter,
-followed by some letters I<and> non-letters/numbers) it is assumed to be an
-option (C<-M>) followed by a value.
-
-=item * If we get something like C<--foo> it is a long option. If the next word is an
-option (starts with a C<->) then it is assumed that this option does not have
-argument. Otherwise, the next word is assumed to be this option's value.
-
-=item * Otherwise, it is an argument (that is, permute is assumed).
-
-=back
-
-This function is not exported by default, but exportable.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<cmdline> => I<str>
-
-Command-line, defaults to COMP_LINE environment.
-
-=item * B<cword> => I<array[str]>
-
-Alternative to passing `cmdline` and `point`.
-
-If you already did a C<parse_cmdline()>, you can pass the cword result (the
-second element) here to avoid calling C<parse_cmdline()> twice.
-
-=item * B<point> => I<int>
-
-Point/position to complete in command-line, defaults to COMP_POINT.
-
-=item * B<words> => I<array[str]>
-
-Alternative to passing `cmdline` and `point`.
-
-If you already did a C<parse_cmdline()>, you can pass the words result (the first
-element) here to avoid calling C<parse_cmdline()> twice.
-
-=back
-
-Returns an enveloped result (an array).
-
-First element (status) is an integer containing HTTP status code
-(200 means OK, 4xx caller error, 5xx function error). Second element
-(msg) is a string containing error message, or 'OK' if status is
-200. Third element (result) is optional, the actual result. Fourth
-element (meta) is called result metadata and is optional, a hash
-that contains extra information.
-
-Return value:  (hash)
-
-
 =head2 point($cmdline, $marker) -> any
 
 Return line with point marked by a marker.
@@ -913,7 +809,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Complete-B
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/sharyanto/perl-Complete-Bash>.
+Source repository is at L<https://github.com/perlancar/perl-Complete-Bash>.
 
 =head1 BUGS
 
@@ -925,11 +821,15 @@ feature.
 
 =head1 SEE ALSO
 
+L<Complete>, the convention that this module uses.
 
-L<Complete>
+Some higher-level modules that use this module (so you don't have to use this
+module directly): L<Getopt::Long::Complete> (via L<Complete::Getopt::Long>),
+L<Getopt::Long::Subcommand>, L<Perinci::CmdLine> (via
+L<Perinci::Sub::Complete>).
 
 Other modules related to bash shell tab completion: L<Bash::Completion>,
-L<Getopt::Complete>. L<Term::Bash::Completion::Generator>
+L<Getopt::Complete>, L<Term::Bash::Completion::Generator>.
 
 Programmable Completion section in Bash manual:
 L<https://www.gnu.org/software/bash/manual/html_node/Programmable-Completion.html>
